##############################################################################
#   Copyright (c) 2018 Eynes/E-MIPS (www.eynes.com.ar)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
##############################################################################

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class OrganizationalUnit(models.Model):
    _name = 'organizational.unit'
    _parent_name = "parent_id"
    _parent_order = 'name'
    _rec_name = 'complete_name'

    name = fields.Char(
        string='Name',
        required=True
    )
    complete_name = fields.Char(
        string="Full Location Name",
        compute='_compute_complete_name',
        store=True
    )
    parent_id = fields.Many2one(
        comodel_name='organizational.unit',
        string='Parent',
        index=True,
        ondelete='cascade'
    )
    level = fields.Integer(
        string='Level',
        compute='_compute_level',
        store=True,
    )
    active = fields.Boolean(string='Active', default=True)

    @api.one
    @api.depends('name', 'parent_id.complete_name')
    def _compute_complete_name(self):
        if self.parent_id.complete_name:
            complete_name = '%s / %s' % (
                self.parent_id.complete_name, self.name
            )
            self.complete_name = complete_name
        else:
            self.complete_name = self.name

    @api.one
    @api.depends('parent_id')
    def _compute_level(self):
        regs = self.search([
            ('id', 'parent_of', self.id),
        ])
        self.level = len(regs)


    @api.constrains('name', 'parent_id')
    def _check_organizational_unit_repeat(self):
        res = self.search([
            ('name', '=', self.name),
            ('parent_id', '=', self.parent_id.id)
        ])
        if len(res) > 1:
            msg = _('There is other Organizational '
                    'Unit with same name and parent')
            raise ValidationError(msg)

    @api.constrains('parent_id')
    def _check_parent_recursion(self):
        if not self._check_recursion():
            raise ValidationError(
                _('Error ! You cannot create recursive categories.')
            )
        return True

    # Lista de todas las OU
    # Busco la primera, si no tiene parent_id la creo
    # por lo que significa que todas las demas no existen.
    # Si existe llamo a este mismo metodo con una lista nueva
    # generada en base a la anterior (quitandole la primer posicion).
    # Asi hasta que se acabe la lista org_unit, por lo que implica
    # que ya consegui toda la cadena.
    # Si en el transcurso no encontre alguna llamo a
    # _create_organizational_unit
    def find_organizational_unit(self, org_unit, last_org_unit=None):
        organizational_unit = org_unit[0]
        ids = self.search(
            [
                ('name', '=', organizational_unit),
                ('parent_id', '=', last_org_unit)
            ]
        )
        if not ids:
            new_ou = self.create({
                'name': organizational_unit,
                'parent_id': last_org_unit
            })
            org_unit.pop(0)
            if not org_unit:
                return new_ou
            res = self.find_organizational_unit(org_unit, new_ou.id)
            return res
        else:
            org_unit.pop(0)
            if not org_unit:
                return ids[0]
            res = self.find_organizational_unit(org_unit, ids[0].id)
            return res
