# -*- coding: utf-8 -*-
###############################################################################
#   Copyright (c) 2019 Eynes/E-MIPS (Gaston Alberto Bertolani)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
###############################################################################

import re
import logging

from odoo import models, api, fields, _
from odoo.exceptions import ValidationError, UserError
from odoo.addons import decimal_precision as dp
from openpyxl import load_workbook
from io import BytesIO
import base64
from datetime import datetime


_logger = logging.getLogger(__name__)

headers = [
    'Nº Personal',
    'ceco',
]


class ImportCecoWizard(models.TransientModel):
    _name = 'import.ceco.wizard'
    _description = "Import CECOs"

    file = fields.Binary(string='File', required=True)

    line_ids = fields.One2many(
        'import.ceco.wizard.line',
        'wizard_id',
        string='Lines'
    )

    is_valid = fields.Boolean(
        string='is_valid',
        default=False
    )

    @api.onchange('file')
    def onchange_file(self):
        self.is_valid = False

    @api.multi
    def write_ceco_recupero(self, employee, ceco):
        employee.write({
            'ceco_recupero_id': ceco.id,
        })
        return True

    @api.multi
    def get_ceco(self, ceco_code):
        ceco_obj = self.env['account.analytic.account']
        ceco = ceco_obj.search([
            ('name', '=', ceco_code),
        ], limit=1)
        return ceco

    @api.multi
    def check_file(self):
        content = self._decrypt_file()
        o_type = self.env.context.get('type', False)
        if o_type == 'create':
            employee_obj = self.env['hr.employee']
            ceco_obj = self.env['account.analytic.account']
        self.write({
            'line_ids': [(5, _, _)]
        })
        flag = True
        header_error = True
        is_empty = True
        lines = []
        repeated_employees = []
        uniq_employees = []
        for index, line in enumerate(content):
            is_empty = False
            if o_type == 'check':
                if header_error:
                    errors = self._check_header(line)
                    if errors:
                        flag = False
                        lines = errors
                        break
                    header_error = False
                if line['Nº Personal'] in uniq_employees:
                    repeated_employees.append(line['Nº Personal'])
                else:
                    uniq_employees.append(line['Nº Personal'])
            elif o_type == 'create':
                employee = employee_obj.search([
                    ('employee_file_id', '=', line['Nº Personal'])
                ])
                ceco = self.get_ceco(line['ceco'])
            for item in line.items():
                if item[0] in headers:
                    if o_type == 'check':
                        error = self._check_content(item, index)
                        if error:
                            flag = False
                            lines.append(error)
                    elif o_type == 'create':
                        self.write_ceco_recupero(employee, ceco)

        if is_empty:
            raise UserError(_('The file is empty'))
        if repeated_employees:
            flag = False
            error_msg = _('The employee with file %s shows up more than once')
            for repeated in repeated_employees:
                lines.append(
                    self._add_line(error_msg % repeated)
                )

        if not flag:
            self.is_valid = False
            self.write({
                'line_ids': lines
            })
            return {
                'context': self.env.context,
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'import.ceco.wizard',
                'res_id': self.id,
                'view_id': False,
                'type': 'ir.actions.act_window',
                'target': 'new',
            }
        else:
            if o_type == 'create':
                return {
                    'type': 'ir.actions.client',
                    'tag': 'reload',
                }
            self.is_valid = True
        return {
            'context': self.env.context,
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'import.ceco.wizard',
            'res_id': self.id,
            'view_id': False,
            'type': 'ir.actions.act_window',
            'target': 'new',
        }

    def _check_header(self, line):
        lines = []
        missing = list(
            filter(lambda x: x not in line.keys(), headers)
        )
        if missing:
            for element in missing:
                lines.append(
                    self._add_line(_('Missing %s column') % element)
                )
        return lines

    def _get_period(self, month):
        period_obj = self.env['date.period']
        annual_plan = self._get_annual_plan()
        year = annual_plan.year
        date = datetime(year, int(month), 1)
        period = period_obj.search([
            ('date_from', '<=', date),
            ('date_to', '>=', date)
        ], limit=1)
        return period

    def _check_content(self, item, index):
        employee_obj = self.env['hr.employee']
        ceco_obj = self.env['account.analytic.account']
        line = index + 2
        if not item[1] or str(item[1]).strip() == '':
            return (
                self._add_line(
                    _('Empty value in %s') % item[0], line
                )
            )

        if item[0] == 'Nº Personal':
            employee = employee_obj.search([
                ('employee_file_id', '=', item[1])
            ], limit=1)
            if not employee:
                return (
                    self._add_line(
                        _('Employee %s not found in the system') % item[1], line
                    )
                )
        if item[0] == 'ceco':
            ceco_code = item[1]
            if ceco_code and not str(ceco_code).isdigit():
                return (
                    self._add_line(
                        _('Invalid CeCo %s. It must be only numbers') % ceco_code, line
                    )
                )
            ceco = self.get_ceco(ceco_code)
            if not ceco:
                return (
                    self._add_line(
                        _('There is not CECO with code %s created.') % ceco_code, line
                    )
                )


    def _decrypt_file(self):
        try:
            file = self.file
            wb = load_workbook(BytesIO(base64.b64decode(file)), data_only=True)
            # sheet = wb.get_sheet_by_name('Input PA')
            sheet = wb.active
            rows = sheet.max_row
            cols = sheet.max_column

            def item(i, j):
                return (
                    sheet.cell(row=1, column=j).value,
                    sheet.cell(row=i, column=j).value
                )
            return (
                dict(item(i, j) for j in range(1, cols + 1))
                for i in range(2, rows + 1)
            )
        except Exception as e:
            if str(e) == 'File is not a zip file':
                raise UserError(_('File must be an Excel file'))
            raise UserError(_(e))

    def _add_line(self, error, line=None):
        try:
            vals = {}
            vals['error'] = error
            vals['line'] = line
            return (0, 0, vals)
        except Exception as e:
            raise UserError(_(e))


class ImportCecoWizardLine(models.TransientModel):
    _name = 'import.ceco.wizard.line'

    wizard_id = fields.Many2one('import.ceco.wizard', readonly=True)
    error = fields.Char(string='Error', readonly=True)
    line = fields.Char(string='Line', readonly=True)
